==> georepo-server-2.6.1-sources.jar: de/narimo/georepo/server/repository/DatasetRepository.java <==
package de.narimo.georepo.server.repository;

import java.io.IOException;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.ws.rs.NotFoundException;

import org.geotools.feature.FeatureCollection;

import de.narimo.commons.jdbc.JDBCConnectionJNDI;
import de.narimo.georepo.server.GeorepoConstants;
import de.narimo.georepo.server.tools.QueryCheck;
import de.narimo.georepo.server.tools.TableTools;

public class DatasetRepository extends ParentDatasetRepository {

    public static FeatureCollection getDatasets(int layerId, long userId) throws SQLException {

        String dataTableName = TableTools.getDataTableName(layerId);
        QueryCheck.checkTableName(dataTableName);

        Map<String, String> columnsAndTypes = getColumnTypes(dataTableName,
                GeorepoConstants.DEFAULTSCHEMA);

        Map<String, Class> propertyTypes = new HashMap<String, Class>();

        for (String colName : columnsAndTypes.keySet()) {
            String colType = columnsAndTypes.get(colName);
            if (colType.equals("integer")) {
                propertyTypes.put(colName, Integer.class);
            } else if (colType.equals("double precision")) {
                propertyTypes.put(colName, Double.class);
            } else {
                propertyTypes.put(colName, String.class);
            }
        }

        Map<Integer, List<String>> tags = null;
        if (checkTagTableExists(layerId)) {
            if (propertyTypes.containsKey("tags")) {
                System.out.println("Warning: Data table already contains a property 'tags'. "
                        + "Cannot match tags from tags table to the dataset.");
            } else {
                columnsAndTypes.put("tags", "text");
                propertyTypes.put("tags", String[].class);
                tags = getTagsByGfid(layerId);
            }
        }

        String query = "SELECT ST_AsText(the_geom), * from public." + dataTableName + ";";

        return createSimpleFeatures(dataTableName, propertyTypes, columnsAndTypes, query, tags);
    }

    public static int getLayerId(String workspace, String featureType) {

        JDBCConnectionJNDI jdbcMeta = null;

        try {
            jdbcMeta = new JDBCConnectionJNDI("jdbc/georepoMetaResource");

            String sql = "SELECT id from public.datasets WHERE workspace = ? AND featuretype = ?;";
            System.out.println(sql);

            PreparedStatement ps = jdbcMeta.prepareStatement(sql);
            ps.setString(1, workspace);
            ps.setString(2, featureType);

            ResultSet rs = ps.executeQuery();

            int count = 0;
            Integer id = null;
            while (rs.next()) {
                id = rs.getInt("id");
                count++;
            }

            if (count > 1) {
                throw new NotFoundException(
                        "Could not determine distinct layer id. Please review datasets table consistency for workspace "
                                + workspace + " and feature type " + featureType + "!");
            }

            return id;
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (jdbcMeta != null) {
                jdbcMeta.closeAll();
                jdbcMeta = null;
            }
        }
        throw new NotFoundException("Could not determine layer id.");
    }

    /**
     * Checks, whether a feature with gfid exists for the given layer.
     * 
     * @param layerId
     * @param gfid
     * @return
     * @throws SQLException
     * @throws IOException
     */
    public static boolean doesFeatureExist(int layerId, int gfid)
            throws SQLException {

        JDBCConnectionJNDI jdbcData = null;

        String dataTable = TableTools.getDataTableName(layerId);

        try {
            jdbcData = new JDBCConnectionJNDI("jdbc/georepoDataResource");

            String sql = "SELECT count(*) FROM " + dataTable + " WHERE gfid = ?;";
            System.out.println(sql);

            PreparedStatement ps = jdbcData.prepareStatement(sql);
            ps.setInt(1, gfid);
            ResultSet rs = ps.executeQuery();

            rs.next();
            int count = rs.getInt("count");
            if (count > 0) {
                return true;
            }

        } catch (Exception e) {
            e.printStackTrace();
            throw new SQLException(e);
        } finally {
            if (jdbcData != null) {
                jdbcData.closeAll();
                jdbcData = null;
            }
        }
        return false;
    }

    public static List<String> getCategories(int layerId, String optCategoryColumn) throws SQLException {

        JDBCConnectionJNDI jdbcData = null;

        String dataTable = TableTools.getDataTableName(layerId);

        List<String> categories = new ArrayList<>();

        String categoryColumn = "category";
        if (optCategoryColumn != null && optCategoryColumn.chars().allMatch(Character::isLetter)) {
            categoryColumn = optCategoryColumn;
        }

        try {
            jdbcData = new JDBCConnectionJNDI("jdbc/georepoDataResource");

            String sql0 = "SELECT column_name FROM information_schema.columns WHERE table_name=? and column_name=?;";
            PreparedStatement ps0 = jdbcData.prepareStatement(sql0);
            ps0.setString(1, dataTable);
            ps0.setString(2, categoryColumn);
            ResultSet rs0 = ps0.executeQuery();
            if (!rs0.next()) {
                return new ArrayList<>();
            }

            String sql = "SELECT DISTINCT " + categoryColumn + " FROM " + dataTable;
            System.out.println(sql);

            PreparedStatement ps = jdbcData.prepareStatement(sql);
            ResultSet rs = ps.executeQuery();

            while (rs.next()) {
                String category = rs.getString(1);
                if (category != null) {
                    categories.add(rs.getString(1));
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
            throw new SQLException(e);
        } finally {
            if (jdbcData != null) {
                jdbcData.closeAll();
                jdbcData = null;
            }
        }
        return categories;
    }

    public static List<String> getTags(int layerId) throws SQLException {

        JDBCConnectionJNDI jdbcData = null;

        String tagTable = TableTools.getTagTableName(layerId);

        List<String> tags = new ArrayList<>();

        try {
            jdbcData = new JDBCConnectionJNDI("jdbc/georepoDataResource");

            String sql = "SELECT DISTINCT tag FROM " + tagTable;
            System.out.println(sql);

            PreparedStatement ps = jdbcData.prepareStatement(sql);
            ResultSet rs = ps.executeQuery();

            while (rs.next()) {
                String tag = rs.getString(1);
                if (tag != null) {
                    tags.add(rs.getString(1));
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
            throw new SQLException(e);
        } finally {
            if (jdbcData != null) {
                jdbcData.closeAll();
                jdbcData = null;
            }
        }
        return tags;
    }

    public static Map<Integer, List<String>> getTagsByGfid(int layerId) throws SQLException {

        JDBCConnectionJNDI jdbcData = null;

        String tagTable = TableTools.getTagTableName(layerId);

        Map<Integer, List<String>> tags = new HashMap<>();

        try {
            jdbcData = new JDBCConnectionJNDI("jdbc/georepoDataResource");

            String sql = "SELECT DISTINCT gfid, tag FROM " + tagTable + " ORDER BY gfid, tag;";
            System.out.println(sql);

            PreparedStatement ps = jdbcData.prepareStatement(sql);
            ResultSet rs = ps.executeQuery();

            while (rs.next()) {
                String tag = rs.getString(2);
                if (tag != null) {
                    int gfid = rs.getInt(1);
                    if (gfid != 0) {
                        List<String> tagList = tags.get(gfid);
                        if (tagList == null) {
                            tagList = new ArrayList<>();
                        }
                        tagList.add(tag);
                        tags.put(gfid, tagList);
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new SQLException(e);
        } finally {
            if (jdbcData != null) {
                jdbcData.closeAll();
                jdbcData = null;
            }
        }
        return tags;
    }

    public static boolean checkTagTableExists(int layerId) throws SQLException {

        JDBCConnectionJNDI jdbcData = null;

        String tagTable = TableTools.getTagTableName(layerId);

        try {
            jdbcData = new JDBCConnectionJNDI("jdbc/georepoDataResource");

            String sql = "SELECT 1 FROM " + tagTable + ";";
            System.out.println(sql);

            jdbcData.executeQuery(sql);

            return true;
        } catch (Exception e) {
            if (e.getMessage().contains("does not exist")) {
                return false;
            }
            e.printStackTrace();
            throw new SQLException(e);
        } finally {
            if (jdbcData != null) {
                jdbcData.closeAll();
                jdbcData = null;
            }
        }
    }

    public static void createTagTable(int layerId) throws SQLException {
        JDBCConnectionJNDI jdbcData = null;

        String tagTable = TableTools.getTagTableName(layerId);

        try {
            jdbcData = new JDBCConnectionJNDI("jdbc/georepoDataResource");

            String sql = "CREATE TABLE public." + tagTable + "("
                    + "id SERIAL PRIMARY KEY,"
                    + "gfid integer NOT NULL,"
                    + "tag text NOT NULL"
                    + ");";

            System.out.println(sql);
            jdbcData.execute(sql);

        } catch (Exception e) {
            e.printStackTrace();
            throw new SQLException(e);
        } finally {
            if (jdbcData != null) {
                jdbcData.closeAll();
                jdbcData = null;
            }
        }
    }
}

==> georepo-server-2.7-sources.jar: de/narimo/georepo/server/repository/DatasetRepository.java <==
package de.narimo.georepo.server.repository;

import java.io.IOException;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.ws.rs.NotFoundException;

import org.geotools.feature.FeatureCollection;

import de.narimo.commons.jdbc.JDBCConnectionJNDI;
import de.narimo.georepo.server.GeorepoConstants;
import de.narimo.georepo.server.tools.QueryCheck;
import de.narimo.georepo.server.tools.TableTools;

public class DatasetRepository extends ParentDatasetRepository {

    public static FeatureCollection getDatasets(int layerId, long userId, boolean asAdmin) throws SQLException {

        String dataTableName = TableTools.getDataTableName(layerId);
        QueryCheck.checkTableName(dataTableName);
        String authTableName = TableTools.getAuthTableName(layerId);
        QueryCheck.checkTableName(authTableName);

        Map<String, String> columnsAndTypes = getColumnTypes(dataTableName,
                GeorepoConstants.DEFAULTSCHEMA);

        Map<String, Class> propertyTypes = new HashMap<String, Class>();

        for (String colName : columnsAndTypes.keySet()) {
            String colType = columnsAndTypes.get(colName);
            if (colType.equals("integer")) {
                propertyTypes.put(colName, Integer.class);
            } else if (colType.equals("double precision")) {
                propertyTypes.put(colName, Double.class);
            } else {
                propertyTypes.put(colName, String.class);
            }
        }

        Map<Integer, List<String>> tags = null;
        if (checkTagTableExists(layerId)) {
            if (propertyTypes.containsKey("tags")) {
                System.out.println("Warning: Data table already contains a property 'tags'. "
                        + "Cannot match tags from tags table to the dataset.");
            } else {
                columnsAndTypes.put("tags", "text");
                propertyTypes.put("tags", String[].class);
                tags = getTagsByGfid(layerId);
            }
        }

        String query = "SELECT ST_AsText(the_geom), * from public." + dataTableName + ";";
        if (asAdmin && checkAuthTableExists(layerId)) {
            columnsAndTypes.put("modifiedat", "text");
            columnsAndTypes.put("modifiedby", "text");
            propertyTypes.put("modifiedat", String.class);
            propertyTypes.put("modifiedby", String.class);
            query = "SELECT ST_AsText(o.the_geom), o.*, a.modifiedby, a.modifiedat from public." + dataTableName
                    + " AS o LEFT JOIN public." + authTableName + " AS a "
                    + " ON o.grpfid = a.grpfid";
        }

        return createSimpleFeatures(dataTableName, propertyTypes, columnsAndTypes, query, tags, asAdmin);
    }

    public static int getLayerId(String workspace, String featureType) {

        JDBCConnectionJNDI jdbcMeta = null;
        Integer id = null;
        int count = 0;
        try {
            jdbcMeta = new JDBCConnectionJNDI("jdbc/georepoMetaResource");

            String sql = "SELECT id from public.datasets WHERE workspace = ? AND featuretype = ?;";
            System.out.println(sql);

            PreparedStatement ps = jdbcMeta.prepareStatement(sql);
            ps.setString(1, workspace);
            ps.setString(2, featureType);

            ResultSet rs = ps.executeQuery();

            while (rs.next()) {
                id = rs.getInt("id");
                count++;
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (jdbcMeta != null) {
                jdbcMeta.closeAll();
                jdbcMeta = null;
            }
        }

        if (count > 1) {
            throw new NotFoundException(
                    "Could not determine distinct layer id. Please review datasets table consistency for workspace "
                            + workspace + " and feature type " + featureType + "!");
        }
        if (id == null) {
            throw new NotFoundException(
                    "Could not determine layer id for workspace "
                            + workspace + " and feature type " + featureType + "!");
        }
        return id;
    }

    /**
     * Checks, whether a feature with gfid exists for the given layer.
     * 
     * @param layerId
     * @param gfid
     * @return
     * @throws SQLException
     * @throws IOException
     */
    public static boolean doesFeatureExist(int layerId, int gfid)
            throws SQLException {

        JDBCConnectionJNDI jdbcData = null;

        String dataTable = TableTools.getDataTableName(layerId);

        try {
            jdbcData = new JDBCConnectionJNDI("jdbc/georepoDataResource");

            String sql = "SELECT count(*) FROM " + dataTable + " WHERE gfid = ?;";
            System.out.println(sql);

            PreparedStatement ps = jdbcData.prepareStatement(sql);
            ps.setInt(1, gfid);
            ResultSet rs = ps.executeQuery();

            rs.next();
            int count = rs.getInt("count");
            if (count > 0) {
                return true;
            }

        } catch (Exception e) {
            e.printStackTrace();
            throw new SQLException(e);
        } finally {
            if (jdbcData != null) {
                jdbcData.closeAll();
                jdbcData = null;
            }
        }
        return false;
    }

    public static List<String> getCategories(int layerId, String optCategoryColumn) throws SQLException {

        JDBCConnectionJNDI jdbcData = null;

        String dataTable = TableTools.getDataTableName(layerId);

        List<String> categories = new ArrayList<>();

        String categoryColumn = "category";
        if (optCategoryColumn != null && optCategoryColumn.chars().allMatch(Character::isLetter)) {
            categoryColumn = optCategoryColumn;
        }

        try {
            jdbcData = new JDBCConnectionJNDI("jdbc/georepoDataResource");

            String sql0 = "SELECT column_name FROM information_schema.columns WHERE table_name=? and column_name=?;";
            PreparedStatement ps0 = jdbcData.prepareStatement(sql0);
            ps0.setString(1, dataTable);
            ps0.setString(2, categoryColumn);
            ResultSet rs0 = ps0.executeQuery();
            if (!rs0.next()) {
                return new ArrayList<>();
            }

            String sql = "SELECT DISTINCT " + categoryColumn + " FROM " + dataTable;
            System.out.println(sql);

            PreparedStatement ps = jdbcData.prepareStatement(sql);
            ResultSet rs = ps.executeQuery();

            while (rs.next()) {
                String category = rs.getString(1);
                if (category != null) {
                    categories.add(rs.getString(1));
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
            throw new SQLException(e);
        } finally {
            if (jdbcData != null) {
                jdbcData.closeAll();
                jdbcData = null;
            }
        }
        return categories;
    }

    public static List<String> getTags(int layerId) throws SQLException {

        JDBCConnectionJNDI jdbcData = null;

        String tagTable = TableTools.getTagTableName(layerId);

        List<String> tags = new ArrayList<>();

        try {
            jdbcData = new JDBCConnectionJNDI("jdbc/georepoDataResource");

            String sql = "SELECT DISTINCT tag FROM " + tagTable;
            System.out.println(sql);

            PreparedStatement ps = jdbcData.prepareStatement(sql);
            ResultSet rs = ps.executeQuery();

            while (rs.next()) {
                String tag = rs.getString(1);
                if (tag != null) {
                    tags.add(rs.getString(1));
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
            throw new SQLException(e);
        } finally {
            if (jdbcData != null) {
                jdbcData.closeAll();
                jdbcData = null;
            }
        }
        return tags;
    }

    public static Map<Integer, List<String>> getTagsByGfid(int layerId) throws SQLException {

        JDBCConnectionJNDI jdbcData = null;

        String tagTable = TableTools.getTagTableName(layerId);

        Map<Integer, List<String>> tags = new HashMap<>();

        try {
            jdbcData = new JDBCConnectionJNDI("jdbc/georepoDataResource");

            String sql = "SELECT DISTINCT gfid, tag FROM " + tagTable + " ORDER BY gfid, tag;";
            System.out.println(sql);

            PreparedStatement ps = jdbcData.prepareStatement(sql);
            ResultSet rs = ps.executeQuery();

            while (rs.next()) {
                String tag = rs.getString(2);
                if (tag != null) {
                    int gfid = rs.getInt(1);
                    if (gfid != 0) {
                        List<String> tagList = tags.get(gfid);
                        if (tagList == null) {
                            tagList = new ArrayList<>();
                        }
                        tagList.add(tag);
                        tags.put(gfid, tagList);
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new SQLException(e);
        } finally {
            if (jdbcData != null) {
                jdbcData.closeAll();
                jdbcData = null;
            }
        }
        return tags;
    }

    public static boolean checkTagTableExists(int layerId) throws SQLException {
        JDBCConnectionJNDI jdbcData = null;
        String tagTable = TableTools.getTagTableName(layerId);

        try {
            jdbcData = new JDBCConnectionJNDI("jdbc/georepoDataResource");

            String sql = "SELECT 1 FROM " + tagTable + ";";
            System.out.println(sql);
            jdbcData.executeQuery(sql);
            return true;
        } catch (Exception e) {
            if (e.getMessage().contains("does not exist")) {
                return false;
            }
            e.printStackTrace();
            throw new SQLException(e);
        } finally {
            if (jdbcData != null) {
                jdbcData.closeAll();
                jdbcData = null;
            }
        }
    }

    public static boolean checkAuthTableExists(int layerId) throws SQLException {
        JDBCConnectionJNDI jdbcData = null;
        String authTable = TableTools.getAuthTableName(layerId);

        try {
            jdbcData = new JDBCConnectionJNDI("jdbc/georepoDataResource");

            String sql = "SELECT 1 FROM " + authTable + ";";
            System.out.println(sql);
            jdbcData.executeQuery(sql);
            return true;
        } catch (Exception e) {
            if (e.getMessage().contains("does not exist")) {
                return false;
            }
            e.printStackTrace();
            throw new SQLException(e);
        } finally {
            if (jdbcData != null) {
                jdbcData.closeAll();
                jdbcData = null;
            }
        }
    }

    /**
     * Add one or multiple tags for a gfid on a layer.
     * 
     * @param layerId
     * @param gfid
     * @param tags
     * @return
     * @throws SQLException
     */
    public static List<String> addTags(int layerId, int gfid, List<String> tags) throws SQLException {

        JDBCConnectionJNDI jdbcData = null;

        String tagTable = TableTools.getTagTableName(layerId);

        try {
            jdbcData = new JDBCConnectionJNDI("jdbc/georepoDataResource");

            StringBuilder sql = new StringBuilder();
            sql.append("INSERT INTO " + tagTable + "(gfid, tag) VALUES");

            for (int cnt = 0; cnt < tags.size(); cnt++) {
                if (cnt > 0) {
                    sql.append(",");
                }
                sql.append("(" + gfid + ",?)");
            }
            sql.append(";");
            System.out.println(sql);

            PreparedStatement ps = jdbcData.prepareStatement(sql.toString());
            int cnt = 1;
            for (String tag : tags) {
                ps.setString(cnt, tag);
                cnt++;
            }
            ps.executeUpdate();

        } catch (Exception e) {
            e.printStackTrace();
            throw new SQLException(e);
        } finally {
            if (jdbcData != null) {
                jdbcData.closeAll();
                jdbcData = null;
            }
        }
        return tags;
    }

    /**
     * Remove one or multiple tags from a gfid on a layer.
     * 
     * @param layerId
     * @param gfid
     * @param tags
     * @return
     * @throws SQLException
     */
    public static List<String> removeTags(int layerId, int gfid, List<String> tags) throws SQLException {

        JDBCConnectionJNDI jdbcData = null;

        String tagTable = TableTools.getTagTableName(layerId);

        try {
            jdbcData = new JDBCConnectionJNDI("jdbc/georepoDataResource");

            StringBuilder sql = new StringBuilder();
            sql.append("DELETE from " + tagTable + " WHERE gfid=" + gfid + " AND (");

            for (int cnt = 0; cnt < tags.size(); cnt++) {
                if (cnt > 0) {
                    sql.append(" OR ");
                }
                sql.append("tag=?");
            }
            sql.append(");");

            System.out.println(sql);

            PreparedStatement ps = jdbcData.prepareStatement(sql.toString());
            int cnt = 1;
            for (String tag : tags) {
                ps.setString(cnt, tag);
                cnt++;
            }
            ps.executeUpdate();

        } catch (Exception e) {
            e.printStackTrace();
            throw new SQLException(e);
        } finally {
            if (jdbcData != null) {
                jdbcData.closeAll();
                jdbcData = null;
            }
        }
        return tags;
    }

    public static void createTagTable(int layerId) throws SQLException {
        JDBCConnectionJNDI jdbcData = null;

        String tagTable = TableTools.getTagTableName(layerId);

        try {
            jdbcData = new JDBCConnectionJNDI("jdbc/georepoDataResource");

            String sql = "CREATE TABLE IF NOT EXISTS public." + tagTable + "("
                    + "id SERIAL PRIMARY KEY,"
                    + "gfid integer NOT NULL,"
                    + "tag text NOT NULL"
                    + ");";

            System.out.println(sql);
            jdbcData.execute(sql);

        } catch (Exception e) {
            e.printStackTrace();
            throw new SQLException(e);
        } finally {
            if (jdbcData != null) {
                jdbcData.closeAll();
                jdbcData = null;
            }
        }
    }

    /**
     * Retrieves the history datasets of a feature for a given gfid ordered by
     * insertedat date descending (newest first).
     * 
     * @param layerId
     * @param userId
     * @return
     * @throws SQLException
     */
    public static FeatureCollection getDatasetHistory(int layerId, long userId, int gfid) throws SQLException {

        if (!ParentDatasetRepository.checkHistoryTableExists(layerId)) {
            // no history datasets available, since we haven't stored any yet
            return null;
        }

        String histTableName = TableTools.getHistTableName(layerId);
        QueryCheck.checkTableName(histTableName);

        Map<String, String> columnsAndTypes = getColumnTypes(histTableName,
                GeorepoConstants.DEFAULTSCHEMA);

        Map<String, Class> propertyTypes = new HashMap<String, Class>();

        for (String colName : columnsAndTypes.keySet()) {
            String colType = columnsAndTypes.get(colName);
            if (colType.equals("integer")) {
                propertyTypes.put(colName, Integer.class);
            } else if (colType.equals("double precision")) {
                propertyTypes.put(colName, Double.class);
            } else {
                propertyTypes.put(colName, String.class);
            }
        }

        String query = "SELECT ST_AsText(the_geom), *, insertedby as insertedbyuser from public." + histTableName
                + " WHERE gfid = " + gfid + " ORDER BY insertedat asc NULLS FIRST;";
        System.out.println(query);

        // we should only allow retrieving history, so we set addUserInformation=true!
        return createSimpleFeatures(histTableName, propertyTypes, columnsAndTypes, query, null, true);
    }

    /**
     * Creates the auth table for the gd_x table
     * 
     * @param authTableName
     * @throws SQLException
     * @throws IOException
     */
    public static void createAuthTable(String authTableName) throws SQLException, IOException {
        JDBCConnectionJNDI jdbcData = null;

        QueryCheck.checkTableName(authTableName);

        try {
            jdbcData = new JDBCConnectionJNDI("jdbc/georepoDataResource");

            String sql = "CREATE TABLE IF NOT EXISTS public." + authTableName
                    + "("
                    + "id SERIAL PRIMARY KEY,"
                    + "grpfid integer UNIQUE NOT NULL,"
                    + "diff_grpfid integer UNIQUE NOT NULL,"
                    + "modifiedby integer NOT NULL,"
                    + "modifiedat timestamp without time zone default '1970-01-01 00:00:01' NOT NULL"
                    + ");";
            System.out.println(sql);

            jdbcData.execute(sql);

        } catch (Exception e) {
            e.printStackTrace();
            throw new SQLException(e);
        } finally {
            if (jdbcData != null) {
                jdbcData.closeAll();
                jdbcData = null;
            }
        }
    }
}
